#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, ItemsView, Iterable, Iterator, KeysView, Optional, Sequence, Type, TypeVar, Tuple, Union, ValuesView
from typing import Any, List, Dict, Set
from typing import cast, overload


#--------------------------------------------------------------------------------
# 제네릭 딕셔너리.
#--------------------------------------------------------------------------------
TKey = TypeVar("TKey", bound = Any)
TValue = TypeVar("TValue", bound = Any)
class Dictionary(Generic[TKey, TValue]):
	#--------------------------------------------------------------------------------
	# 멤버 변수 목록.
	#--------------------------------------------------------------------------------
	__dictionary: dict[TKey, TValue]


	#--------------------------------------------------------------------------------
	# 생성됨.
	#--------------------------------------------------------------------------------
	def __init__(self) -> None:
		self.__dictionary = dict()


	#--------------------------------------------------------------------------------
	# 값 넣기.
	#--------------------------------------------------------------------------------
	def __setitem__(self, key: TKey, value: TValue) -> None:
		self.__dictionary[TKey] = value


	#--------------------------------------------------------------------------------
	# 값 반환.
	#--------------------------------------------------------------------------------
	def __getitem__(self, key: TKey) -> TValue:
		if key not in self.__dictionary:
			raise KeyError(key)
		return self.__dictionary[key]


	#--------------------------------------------------------------------------------
	# 클래스 시리얼라이즈.
	#--------------------------------------------------------------------------------
	def __repr__(self) -> str:
		return repr(self.__dictionary)


	#--------------------------------------------------------------------------------
	# 키 목록 반환.
	#--------------------------------------------------------------------------------
	def keys(self) -> KeysView[TKey]:
		return self.__dictionary.keys()


	#--------------------------------------------------------------------------------
	# 값 목록 반환.
	#--------------------------------------------------------------------------------
	def values(self) -> ValuesView[TValue]:
		return self.__dictionary.values()


	#--------------------------------------------------------------------------------
	# 요소 반환.
	#--------------------------------------------------------------------------------
	def items(self) -> ItemsView[TKey, TValue]:
		return self.__dictionary.items()


	#--------------------------------------------------------------------------------
	# 갯수 반환.
	#--------------------------------------------------------------------------------
	def __len__(self) -> int:
		return len(self.__dictionary)


	#--------------------------------------------------------------------------------
	# 포함 여부 반환.
	#--------------------------------------------------------------------------------
	def __contains__(self, key: TKey) -> bool:
		return key in self.__dictionary